@@ -1,3 +1,4 @@
+/* -*- Mode: C; c-file-style: "bsd" -*- */
 /* keylist.c - Linked key lists
  *        Copyright (C) 2002 Timo Schulz
  *        Copyright (C) 1998-2002 Free Software Foundation, Inc.
@@ -102,6 +103,7 @@ pklist_select_algo (CDK_KEYLIST pkl, int preftype)
 	prefs = pkr->key.pk->uid->prefs;
       else
 	prefs = pkr->key.pk->prefs;
+      
       any = 0;
       for (i = 0; prefs && prefs[i].type; i++)
 	{
@@ -160,7 +162,7 @@ pklist_select_algo (CDK_KEYLIST pkl, int preftype)
 	    i = 1; /* yep; we can use compression algo 1 */
 	}
     }
-
+  _cdk_log_debug ("selected algo %d from prefs\n", i);
   return i;
 }
 
@@ -256,7 +258,7 @@ cdk_pklist_build (CDK_KEYDB_HD hd, CDK_STRLIST remusr,
 	  pkl = r;
 	}
     }
-leave:
+ leave:
   if (rc)
     {
       cdk_pklist_release (pkl);
@@ -273,7 +275,7 @@ cdk_pklist_encrypt (CDK_KEYLIST pkl, CDK_DEK dek, CDK_STREAM outp)
   cdkPKT_public_key * pk = NULL;
   cdkPKT_pubkey_enc * enc = NULL;
   CDK_PACKET pkt;
-  GCRY_MPI frame = NULL;
+  CDK_SESSKEY frame = NULL;
   int nbits = 0;
   int rc = 0;
 
@@ -293,16 +295,16 @@ cdk_pklist_encrypt (CDK_KEYLIST pkl, CDK_DEK dek, CDK_STREAM outp)
       enc->pubkey_algo = pk->pubkey_algo;
       cdk_pk_get_keyid (pk, enc->keyid);
       nbits = cdk_pk_get_nbits (pk);
-      rc = _cdk_pkcs1_sesskey (&frame, dek, nbits);
+      rc = cdk_dek_encode_pkcs1( dek, nbits, &frame );
       if (rc)
         goto fail;
-      rc = cdk_pk_encrypt (pk, enc, frame);
-      gcry_mpi_release (frame);
-      if (rc)
+      rc = cdk_pk_encrypt( pk, enc, frame );
+      cdk_sesskey_free( frame );
+      if( rc )
         goto fail;
       else
 	{
-	  init_packet (&pkt);
+	  cdk_pkt_init (&pkt);
           pkt.old_ctb = dek->rfc1991? 1 : 0;
 	  pkt.pkttype = CDK_PKT_PUBKEY_ENC;
 	  pkt.pkt.pubkey_enc = enc;
@@ -360,7 +362,7 @@ cdk_sklist_build (CDK_KEYDB_HD hd, CDK_STRLIST locusr,
 	{
           if (unlock)
             {
-              rc = _cdk_seckey_unprotect2 (sk);
+              rc = _cdk_sk_unprotect_auto (sk);
               if (rc)
                 goto fail;
             }
@@ -369,7 +371,6 @@ cdk_sklist_build (CDK_KEYDB_HD hd, CDK_STRLIST locusr,
 	    return CDK_Out_Of_Core;
 	  r->key.sk = sk;
 	  r->next = skl;
-	  r->mark = 0;
 	  r->type = CDK_PKT_SECRET_KEY;
 	  skl = r;
 	}
@@ -395,7 +396,7 @@ cdk_sklist_build (CDK_KEYDB_HD hd, CDK_STRLIST locusr,
 	    {
               if (unlock)
                 {
-                  rc = _cdk_seckey_unprotect2 (sk);
+                  rc = _cdk_sk_unprotect_auto (sk);
                   if (rc)
                     goto fail; 
                 }
@@ -404,7 +405,6 @@ cdk_sklist_build (CDK_KEYDB_HD hd, CDK_STRLIST locusr,
 		return CDK_Out_Of_Core;
 	      r->key.sk = sk;
 	      r->next = skl;
-	      r->mark = 0;
               r->type = CDK_PKT_SECRET_KEY;
 	      skl = r;
 	    }
@@ -458,7 +458,7 @@ cdk_sklist_write_onepass (CDK_KEYLIST skl, CDK_STREAM outp, int sigclass)
       ops->pubkey_algo = r->key.sk->pubkey_algo;
       ops->last = (skcount == 1);
 
-      init_packet (&pkt);
+      cdk_pkt_init (&pkt);
       pkt.pkttype = CDK_PKT_ONEPASS_SIG;
       pkt.pkt.onepass_sig = ops;
       rc = cdk_pkt_build (outp, &pkt);
@@ -472,22 +472,23 @@ cdk_sklist_write_onepass (CDK_KEYLIST skl, CDK_STREAM outp, int sigclass)
 
 
 int
-cdk_sklist_write (CDK_KEYLIST skl, CDK_STREAM outp, GCRY_MD_HD hash,
+cdk_sklist_write (CDK_KEYLIST skl, CDK_STREAM outp, void * mdctx,
 		  int sigclass, int sigver)
 {
   CDK_KEYLIST r = NULL;
   cdkPKT_signature *sig = NULL;
   CDK_PACKET pkt;
-  GCRY_MD_HD md = NULL;
+  GCRY_MD_HD hash, md = NULL;
   byte digest[24]; /* maximal 192-bit for TIGER-192 */
   int rc = 0;
 
-  if (!skl || !outp || !hash)
+  if( !skl || !outp || !mdctx )
     return CDK_Inv_Value;
 
   if (skl->type != CDK_PKT_SECRET_KEY)
     return CDK_Inv_Mode;
-  
+
+  hash = (GCRY_MD_HD)mdctx;
   for (r = skl; r; r = r->next)
     {
       sig = cdk_calloc (1, sizeof *sig);
@@ -505,7 +506,7 @@ cdk_sklist_write (CDK_KEYLIST skl, CDK_STREAM outp, GCRY_MD_HD hash,
       rc = cdk_pk_sign (r->key.sk, sig, digest);
       if (rc)
         goto fail;
-      init_packet (&pkt);
+      cdk_pkt_init (&pkt);
       pkt.old_ctb = sig->version == 3? 1 : 0;
       pkt.pkttype = CDK_PKT_SIGNATURE;
       pkt.pkt.signature = sig;
