@@ -1,3 +1,4 @@
+/* -*- Mode: C; c-file-style: "bsd" -*- */
 /* main.c
  *       Copyright (C) 2001, 2002 Timo Schulz 
  *
@@ -57,10 +58,9 @@ static CDK_PASSPHRASE_CB passphrase_cb = NULL;
 const char *
 cdk_strerror (int ec)
 {
-  static char buf[20];
+    static char buf[20];
 
-  switch (ec)
-    {
+    switch (ec) {
     case CDK_EOF:              return "End Of File";
     case CDK_Success:          return "No error";
     case CDK_General_Error:    return "General error";
@@ -88,14 +88,14 @@ cdk_strerror (int ec)
     case CDK_Too_Short:        return "Buffer or object is too short";
     default:                   sprintf (buf, "ec=%d", ec); return buf;
     }
-  return NULL;
+    return NULL;
 }
 
 
 static void
 out_of_core (size_t n)
 {
-  fprintf (stderr, "\n ** fatal error: out of memory (%d bytes) **\n", n);
+    fprintf (stderr, "\n ** fatal error: out of memory (%d bytes) **\n", n);
 }
 
 
@@ -115,29 +115,29 @@ cdk_set_malloc_hooks (void *(*new_alloc_func) (size_t n),
 		      void *(*new_calloc_func) (size_t m, size_t n),
 		      void (*new_free_func) (void *))
 {
-  alloc_func = new_alloc_func;
-  alloc_secure_func = new_alloc_secure_func;
-  realloc_func = new_realloc_func;
-  calloc_func = new_calloc_func;
-  free_func = new_free_func;
-  malloc_hooks = 1;
+    alloc_func = new_alloc_func;
+    alloc_secure_func = new_alloc_secure_func;
+    realloc_func = new_realloc_func;
+    calloc_func = new_calloc_func;
+    free_func = new_free_func;
+    malloc_hooks = 1;
 }
 
 
 int
 cdk_malloc_hook_initialized (void)
 {
-  return malloc_hooks;
+    return malloc_hooks;
 }
 
 
 void *
 cdk_malloc (size_t size)
 {
-  void * p = alloc_func (size);
-  if (!p)
-    out_of_core (size);
-  return p;
+    void * p = alloc_func (size);
+    if (!p)
+        out_of_core (size);
+    return p;
 }
 
 
@@ -154,18 +154,17 @@ cdk_calloc (size_t n, size_t m)
 static void
 _secmem_init (size_t size)
 {
-  if (!size)
-    {
-      gcry_control (GCRYCTL_DROP_PRIVS);
-      return;
+    if (!size) {
+        gcry_control (GCRYCTL_DROP_PRIVS);
+        return;
     }
-  if (secmem_init == 1)
-    return;
-  if (size >= 65536)
-    size = 16384;
-  gcry_control (GCRYCTL_INIT_SECMEM, size, 0);
-  gcry_control (GCRYCTL_DISABLE_SECMEM_WARN);
-  secmem_init = 1;
+    if (secmem_init == 1)
+        return;
+    if (size >= 65536)
+        size = 16384;
+    gcry_control (GCRYCTL_INIT_SECMEM, size, 0);
+    gcry_control (GCRYCTL_DISABLE_SECMEM_WARN);
+    secmem_init = 1;
 }
 
 
@@ -178,65 +177,55 @@ _secmem_end (void)
 
 
 void *
-cdk_salloc (size_t size)
+cdk_salloc (size_t size, int clear)
 {
-  static size_t n = 0;
-  void * p;
+    static size_t n = 0;
+    void * p;
   
-  if (!secmem_init)
-    {
-      _secmem_init (32768);
-      secmem_init = 1;
+    if (!secmem_init) {
+        _secmem_init (32768);
+        secmem_init = 1;
     }
-  if (secmem_init == 1)
-    {
-      _secmem_init (0);
-      secmem_init = 2;
+    if (secmem_init == 1) {
+        _secmem_init (0);
+        secmem_init = 2;
     }
-  n += size;
-  /*_cdk_log_debug ("\ncdk_salloc (%d)\n", n);*/
-  p = alloc_secure_func (size);
-  if (!p)
-    out_of_core (size);
-
-  return p;
-}
-
-
-void *
-cdk_scalloc (size_t m)
-{
-  void * p = cdk_salloc (m);
-  memset (p, 0, m);
-  return p;
+    n += size;
+    /*_cdk_log_debug ("\ncdk_salloc (%d)\n", n);*/
+    p = alloc_secure_func (size);
+    if (!p)
+        out_of_core (size);
+    if (clear)
+        memset (p, 0, size);
+    return p;
 }
 
 
 void *
 cdk_realloc (void *ptr, size_t size)
 {
-  void * p = realloc_func (ptr, size);
-  if (!p)
-    out_of_core (size);
-  return p;
+    void * p = realloc_func (ptr, size);
+    if (!p)
+        out_of_core (size);
+    return p;
 }
 
 
 char *
 cdk_strdup (const char * ptr)
 {
-  char * p = cdk_malloc (strlen (ptr) + 1);
-  if (p)
-    strcpy (p, ptr);
-  return p;
+    char * p = cdk_malloc (strlen (ptr) + 1);
+    if (p)
+        strcpy (p, ptr);
+    return p;
 }
 
 
 void
 cdk_free (void * ptr)
 {
-  if (ptr)
-    free_func (ptr);
+    if (ptr)
+        free_func (ptr);
 }
 
 
@@ -244,17 +233,15 @@ static void
 _cdk_logv (int level, const char *fmt, va_list arg_ptr)
 {
 
-  if (log_handler)
-    log_handler (log_handler_value, level, fmt, arg_ptr);
-  else
-    {
-      switch (level)
-        {
+    if (log_handler)
+        log_handler (log_handler_value, level, fmt, arg_ptr);
+    else {
+        switch (level) {
         case CDK_LOG_INFO : break;
         case CDK_LOG_DEBUG: fputs ("DBG: ", stderr); break;
         case CDK_LOG_NONE : return; 
         }
-      vfprintf (stderr, fmt, arg_ptr); 
+        vfprintf (stderr, fmt, arg_ptr); 
     }
 }
 
@@ -268,8 +255,8 @@ _cdk_logv (int level, const char *fmt, va_list arg_ptr)
 void
 cdk_set_log_handler (CDK_LOG_FNC logfnc, void * opaque)
 {
-  log_handler = logfnc;
-  log_handler_value = opaque; 
+    log_handler = logfnc;
+    log_handler_value = opaque; 
 }
 
 
@@ -288,41 +275,41 @@ cdk_set_log_level (int lvl)
 int
 _cdk_get_log_level (void)
 {
-  return log_level;
+    return log_level;
 }
 
 
 void
 _cdk_log_info (const char *fmt, ...)
 {
-  va_list arg;
+    va_list arg;
 
-  if (log_level == CDK_LOG_NONE)
-    return;
-  va_start (arg, fmt);
-  _cdk_logv (CDK_LOG_INFO, fmt, arg);
-  va_end (arg);
+    if (log_level == CDK_LOG_NONE)
+        return;
+    va_start (arg, fmt);
+    _cdk_logv (CDK_LOG_INFO, fmt, arg);
+    va_end (arg);
 }
 
 
 void
 _cdk_log_debug (const char *fmt, ...)
 {
-  va_list arg;
+    va_list arg;
   
-  if (log_level < CDK_LOG_DEBUG)
-    return;
-  va_start (arg, fmt);
-  _cdk_logv (CDK_LOG_DEBUG, fmt, arg);
-  va_end (arg);
+    if (log_level < CDK_LOG_DEBUG)
+        return;
+    va_start (arg, fmt);
+    _cdk_logv (CDK_LOG_DEBUG, fmt, arg);
+    va_end (arg);
 }
 
 
 void
 cdk_passphrase_cb_set (CDK_PASSPHRASE_CB pass_fnc)
 {
-  if (pass_fnc)
-    passphrase_cb = pass_fnc;
+    if (pass_fnc)
+        passphrase_cb = pass_fnc;
 }
 
 
@@ -330,7 +317,7 @@ cdk_passphrase_cb_set (CDK_PASSPHRASE_CB pass_fnc)
 char *
 getpass (const char * prompt)
 {
-  return NULL; /* fixme */
+    return NULL; /* fixme */
 }
 #endif
 
@@ -338,37 +325,180 @@ getpass (const char * prompt)
 char *
 _cdk_passphrase_get (const char * prompt)
 {
-  if (passphrase_cb)
-    return passphrase_cb (prompt);
-  return getpass (prompt);
+    if (passphrase_cb)
+        return passphrase_cb (prompt);
+    return getpass (prompt);
 }
 
 
 void
 _cdk_passphrase_free (char *pw)
 {
-  if (passphrase_cb)
-    {
-      cdk_free (pw);
-      return;
+    if (passphrase_cb) {
+        cdk_free (pw);
+        return;
     }
-  /* if we use getpass, we've to free the handle with free() because
-     it was *not* allocated with the gcry memory routines */
-  free (pw);
+    /* if we use getpass, we've to free the handle with free() because
+       it was *not* allocated with the gcry memory routines */
+    free (pw);
 }
 
 
 int
 _cdk_is_idea_available (void)
 {
-  int rc = 0;
-  #ifdef LIBGCRYPT_WITH_IDEA
-  rc = 1;
-  #endif
-  return rc;
+    int rc = 0;
+#ifdef LIBGCRYPT_WITH_IDEA
+    rc = 1;
+#endif
+    return rc;
 }
 
 
+static void
+handle_set_cipher (CDK_HD hd, int cipher)
+{
+    if( !hd )
+        return;
+    if (_cdk_cipher_test_algo (cipher))
+        cipher = GCRY_CIPHER_CAST5;
+    hd->cipher_algo = cipher;   
+}
+
+
+static void
+handle_set_digest (CDK_HD hd, int digest)
+{
+    if( !hd )
+        return;
+    if( _cdk_md_test_algo( digest ) )
+        digest = GCRY_MD_SHA1;
+    hd->digest_algo = digest;   
+}
+
+
+static void
+handle_set_s2k (CDK_HD hd, int mode, int digest, int cipher)
+{
+    if( !hd )
+        return;
+    if( _cdk_cipher_test_algo( cipher ) )
+        cipher = GCRY_CIPHER_CAST5;
+    if( _cdk_md_test_algo( digest ) )
+        digest = GCRY_MD_SHA1;
+    if( mode < 0 || mode > 3 )
+        mode = 3;
+    hd->_s2k.mode = mode;
+    hd->_s2k.digest_algo = digest;
+    hd->_s2k.cipher_algo = cipher;    
+}
+
+
+static void
+handle_set_compat( CDK_HD hd, int val )
+{
+    if( !hd )
+        return;
+    hd->opt.compat = val;
+    if( !val )
+        return;
+    hd->opt.mdc = 0;
+    hd->opt.rfc1991 = val == -1? 1: 0;
+    hd->compress.algo = CDK_COMPRESS_ZIP;
+    hd->compress.level = -1;
+    hd->cipher_algo = val == -1? GCRY_CIPHER_IDEA : GCRY_CIPHER_CAST5;
+    hd->digest_algo = val == -1? GCRY_MD_MD5: GCRY_MD_SHA1;
+    if( val == -1 )
+        handle_set_s2k( hd, 0, hd->digest_algo, hd->cipher_algo );   
+}
+
+
+static void
+handle_set_compress( CDK_HD hd, int algo, int level )
+{
+    if( !hd )
+        return;
+    if( algo < 0 || algo > 2 )
+        algo = 0;
+    hd->compress.algo = algo;
+    if( !algo )
+        hd->opt.compress = 0;
+    else {
+        if( level > 0 && level < 10 )
+            hd->compress.level = level;
+        else
+            hd->compress.level = 6;
+    }
+}
+
+
+int
+cdk_handle_control( CDK_HD hd, int action, int cmd, ... )
+{
+    va_list arg_ptr;
+    int set = action == CDK_CTLF_SET, val = 0;
+    
+    if( action != CDK_CTLF_SET && action != CDK_CTLF_GET )
+        return -1;
+    va_start( arg_ptr, cmd );
+    switch( cmd ) {
+    case CDK_CTL_ARMOR:
+        if( set )
+            hd->opt.armor = va_arg( arg_ptr, int );
+        else
+            val = hd->opt.armor;
+        break;
+
+    case CDK_CTL_CIPHER:
+        if( set )
+            handle_set_cipher( hd, va_arg( arg_ptr, int ) );
+        else
+            val = hd->cipher_algo;
+        break;
+
+    case CDK_CTL_DIGEST:
+        if( set )
+            handle_set_digest( hd, va_arg( arg_ptr, int ) );
+        else
+            val = hd->digest_algo;
+        break;
+
+    case CDK_CTL_COMPAT:
+        if( set )
+            handle_set_compat( hd, va_arg( arg_ptr, int ) );
+        else
+            val = hd->opt.compat;
+        break;
+
+    case CDK_CTL_OVERWRITE:
+        if( set )
+            hd->opt.overwrite = va_arg( arg_ptr, int );
+        else
+            val = hd->opt.overwrite;
+        break;
+
+    case CDK_CTL_COMPRESS:
+        if( set )
+            handle_set_compress( hd, va_arg( arg_ptr, int ),
+                                 va_arg( arg_ptr, int ) );
+        else
+            val = hd->compress.algo;
+        break;
+
+    case CDK_CTL_S2K:
+        if( set )
+            handle_set_s2k( hd, va_arg( arg_ptr, int ), va_arg( arg_ptr, int ),
+                            va_arg( arg_ptr, int ) );
+        else
+            val = hd->_s2k.mode;
+        break;
+    }
+    va_end( arg_ptr );
+    return val;
+}
+
+            
+
 /**
  * cdk_handle_new: create a new session handle
  * @r_ctx: context to store the handle
@@ -377,135 +507,31 @@ _cdk_is_idea_available (void)
 int
 cdk_handle_new (CDK_HD * r_ctx)
 {
-  CDK_HD c;
+    CDK_HD c;
   
-  if (!r_ctx)
-    return CDK_Inv_Value;
+    if (!r_ctx)
+        return CDK_Inv_Value;
   
-  c = cdk_calloc (1, sizeof *c);
-  if (!c)
-    return CDK_Out_Of_Core;
-  /* default */
-  c->_s2k.mode = 3;
-  c->_s2k.digest_algo = GCRY_MD_SHA1;
-  c->_s2k.cipher_algo = GCRY_CIPHER_CAST5;
+    c = cdk_calloc (1, sizeof *c);
+    if (!c)
+        return CDK_Out_Of_Core;
+    /* default */
+    c->_s2k.mode = 3;
+    c->_s2k.digest_algo = GCRY_MD_SHA1;
+    c->_s2k.cipher_algo = GCRY_CIPHER_CAST5;
 
-  c->opt.mdc = 1;
-  c->opt.compress = 1;
-  c->opt.armor = 0;
-  c->opt.textmode = 0;
+    c->opt.mdc = 1;
+    c->opt.compress = 1;
+    c->opt.armor = 0;
+    c->opt.textmode = 0;
 
-  c->digest_algo = GCRY_MD_SHA1;
-  c->cipher_algo = GCRY_CIPHER_CAST5;
-  c->compress_algo = CDK_COMPRESS_ZIP;
+    c->digest_algo = GCRY_MD_SHA1;
+    c->cipher_algo = GCRY_CIPHER_CAST5;
+    c->compress.algo = CDK_COMPRESS_ZIP;
+    c->compress.level = 6;
 
-  *r_ctx = c;
-  return 0;
-}
-
-
-/**
- * cdk_handle_set_s2k: set a string to key mode
- * @hd: the session handle
- * @mode: the s2k mode
- * @digest: the s2k digest algo
- * @cipher: the s2k cipher algo
- *
- **/
-void
-cdk_handle_set_s2k (CDK_HD hd, int mode, int digest, int cipher)
-{
-  if (hd)
-    {
-      if (_cdk_cipher_test_algo (cipher))
-        cipher = GCRY_CIPHER_CAST5;
-      if (_cdk_md_test_algo (digest))
-        digest = GCRY_MD_SHA1;
-      if (mode < 0 || mode > 3)
-        mode = 3;
-      hd->_s2k.mode = mode;
-      hd->_s2k.digest_algo = digest;
-      hd->_s2k.cipher_algo = cipher;
-    }
-}
-
-
-/**
- * cdk_handle_set_cipher: set the cipher algorithm
- * @hd: the session handle
- * @cipher: cipher algorithm
- *
- * This is only used for symmetric encryption only because otherwise
- * the algorithm is extracted from the prefs.
- **/
-void
-cdk_handle_set_cipher (CDK_HD hd, int cipher)
-{
-  if (hd)
-    {
-      if (_cdk_cipher_test_algo (cipher))
-        cipher = GCRY_CIPHER_CAST5;
-      hd->cipher_algo = cipher;
-    }
-}
-
-
-/**
- * cdk_handle_set_digest: set message digest algorithm
- * @hd: the session handle
- * @digest: the message digest algorithm
- *
- **/
-void
-cdk_handle_set_digest (CDK_HD hd, int digest)
-{
-  if (hd)
-    {
-      if (_cdk_md_test_algo (digest))
-        digest = GCRY_MD_SHA1;
-      hd->digest_algo = digest;
-    }
-}
-
-
-/**
- * cdk_handle_set_compress: set the compression algorithm
- * @hd: session handle
- * @algo: compression algo
- *
- **/
-void
-cdk_handle_set_compress (CDK_HD hd, int algo)
-{
-  if (hd)
-    {
-      if (algo < 0 || algo > 2)
-        algo = 0;
-      hd->compress_algo = algo;
-      if (!algo)
-        hd->opt.compress = 0;
-    }
-}
-
-
-/**
- * cdk_handle_set_armor: en- or disable ascii armor
- * @hd: session handle
- * @val: boolean value (1=yes, 0=no)
- *
- **/
-void
-cdk_handle_set_armor (CDK_HD hd, int val)
-{
-  if (hd)
-    hd->opt.armor = val;
-}
-
-
-int
-cdk_handle_get_armor (CDK_HD hd)
-{
-  return hd->opt.armor;
+    *r_ctx = c;
+    return 0;
 }
 
 
@@ -520,40 +546,25 @@ cdk_handle_get_armor (CDK_HD hd)
 void
 cdk_handle_set_keydb (CDK_HD hd, CDK_KEYDB_HD db)
 {
-  if (hd)
-    {
-      if (db->secret)
+    if( !hd )
+        return;
+    if (db->secret)
         hd->db.sec = db;
-      else
+    else
         hd->db.pub = db;
-    }
 }
 
 
-/**
- * cdk_handle_set_compat: set compatibility mode for PGP.
- * @hd: session handle
- * @val: boolean value (1=yes, 0=no, -1=pgp2)
- *
- * If you enable this mode, only algorithms are used which are part
- * of older PGP versions.
- **/
-void
-cdk_handle_set_compat (CDK_HD hd, int val)
+CDK_KEYDB_HD
+cdk_handle_get_keydb( CDK_HD hd, int type )
 {
-  if (hd)
-    {
-      hd->opt.compat = val;
-      if (!val)
-        return;
-      hd->opt.mdc = 0;
-      hd->opt.rfc1991 = val == -1? 1: 0;
-      hd->compress_algo = CDK_COMPRESS_ZIP;
-      hd->cipher_algo = val == -1? GCRY_CIPHER_IDEA : GCRY_CIPHER_CAST5;
-      hd->digest_algo = val == -1? GCRY_MD_MD5: GCRY_MD_SHA1;
-      if (val == -1)
-        cdk_handle_set_s2k (hd, 0, hd->digest_algo, hd->cipher_algo);
-    }
+    if( !hd )
+        return NULL;
+    if( type == CDK_DBTYPE_PK_KEYRING )
+        return hd->db.pub;
+    else if( type == CDK_DBTYPE_SK_KEYRING )
+        return hd->db.sec;
+    return NULL;
 }
 
 
@@ -569,11 +580,10 @@ cdk_handle_set_callback (CDK_HD hd,
                          void (*cb) (void * opaque, int type, const char * s),
                          void * cb_value)
 {
-  if (hd)
-    {
-      hd->callback = cb;
-      hd->callback_value = cb_value;
-    }
+    if( !hd )
+        return;
+    hd->callback = cb;
+    hd->callback_value = cb_value;
 }
 
 
@@ -585,12 +595,11 @@ cdk_handle_set_callback (CDK_HD hd,
 void
 cdk_handle_free (CDK_HD hd)
 {
-  if (hd)
-    {
-      _cdk_result_verify_free (hd->result.verify);
-      cdk_free (hd->s2k);
-      cdk_free (hd->dek);
-      cdk_free (hd);
-    }
+    if( !hd )
+        return;
+    _cdk_result_verify_free (hd->result.verify);
+    cdk_free (hd->s2k);
+    cdk_free (hd->dek);
+    cdk_free (hd);
 }
 
