@@ -1,3 +1,4 @@
+/* -*- Mode: C; c-file-style: "bsd" -*- */
 /* main.h
  *       Copyright (C) 2002 Timo Schulz
  *
@@ -21,6 +22,7 @@
 #ifndef CDK_MAIN_H
 #define CDK_MAIN_H
 
+#include <gcrypt.h>
 #include "types.h"
 #include "context.h"
 
@@ -35,8 +37,6 @@
 #define KEY_CAN_ENCRYPT(a) (_cdk_pk_algo_usage((a)) & GCRY_PK_USAGE_ENCR)
 #define KEY_CAN_SIGN(a)    (_cdk_pk_algo_usage((a)) & GCRY_PK_USAGE_SIGN)
 
-#define init_packet(ctx) memset ((ctx), 0, sizeof (ctx))
-
 #define wipemem(_ptr,_len) \
 do \
 { \
@@ -62,16 +62,19 @@ void _cdk_passphrase_free (char * pw);
 int _cdk_is_idea_available (void);
 
 /*-- misc.c --*/
+int _cdk_check_file( const char * file );
 u32 _cdk_timestamp (void);
 int _cdk_strcmp (const char * a, const char * b);
 u32 _cdk_buftou32 (const byte * buf);
 void _cdk_u32tobuf (u32 u, byte * buf);
-const char* _cdk_memistr (const char * buf, size_t buflen, const char * sub);
+const char * _cdk_memistr (const char * buf, size_t buflen, const char * sub);
 
 /*-- pubkey.c --*/
+u32 _cdk_pkt_get_keyid( CDK_PACKET * pkt, u32 * keyid );
+int _cdk_fingerprint_get (CDK_PACKET * pkt, byte * fpr);
 int _cdk_pk_algo_usage (int algo);
 int _cdk_pk_test_algo (int algo, unsigned int usage);
-int _cdk_seckey_get_csum (cdkPKT_secret_key * sk);
+int _cdk_sk_get_csum (cdkPKT_secret_key * sk);
 
 /*-- cipher.c --*/
 int _cdk_cipher_test_algo (int algo);
@@ -82,36 +85,26 @@ byte * _cdk_subpkt_get_array (CDK_SUBPKT s, int count, size_t * r_nbytes);
 int _cdk_subpkt_add (CDK_SUBPKT root, CDK_SUBPKT node);
 int _cdk_subpkt_copy (CDK_SUBPKT * r_dst, CDK_SUBPKT src);
 int _cdk_subpkt_hash (CDK_SUBPKT hashed, size_t * r_nbytes, GCRY_MD_HD hd);
+void _cdk_subpkt_init( CDK_SUBPKT node, int type, const void * buf,
+                       size_t buflen );
 
 /*-- sig-check.c --*/
-void _cdk_hash_pubkey (cdkPKT_public_key * pk, GCRY_MD_HD md, int use_fpr);
-int _cdk_signature_check (cdkPKT_public_key * pk, cdkPKT_signature *sig,
-                          GCRY_MD_HD digest, int * r_expired);
+int _cdk_sig_check (cdkPKT_public_key * pk, cdkPKT_signature *sig,
+                    GCRY_MD_HD digest, int * r_expired);
 void _cdk_hash_sig_data (cdkPKT_signature * sig, GCRY_MD_HD hd);
 void _cdk_hash_userid (cdkPKT_user_id * uid, int sig_version, GCRY_MD_HD md);
-
+void _cdk_hash_pubkey (cdkPKT_public_key * pk, GCRY_MD_HD md, int use_fpr);
 
 /*-- kbnode.c --*/
 void _cdk_kbnode_add (CDK_KBNODE root, CDK_KBNODE node);
 void _cdk_kbnode_clone (CDK_KBNODE node);
 
 /*-- sesskey.c --*/
-int _cdk_pkcs1_sesskey (GCRY_MPI * esk, CDK_DEK dek, int nbits);
-int _cdk_pkcs1_digest (byte ** r_md, size_t * r_mdlen, int pk_algo,
-                       const byte * md, int digest_algo, unsigned nbits);
-int _cdk_seckey_unprotect2 (cdkPKT_secret_key * sk);
-
-/*-- seckey.c --*/
-int cdk_hash_passphrase (CDK_DEK dek, char * pw, CDK_S2K_HD s2k, int creat);
+int _cdk_digest_encode_pkcs1( byte ** r_md, size_t * r_mdlen, int pk_algo,
+                              const byte * md, int digest_algo, unsigned nbits );
+int _cdk_sk_unprotect_auto (cdkPKT_secret_key * sk);
 
 /*-- keydb.c --*/
-u32 _cdk_keyid_get (CDK_PACKET * pkt, u32 * keyid);
-int _cdk_fingerprint_get (CDK_PACKET * pkt, byte * fpr);
-     
-CDK_KBNODE _cdk_keydb_get_pkblock (CDK_KEYDB_HD hd, u32 * keyid);
-CDK_KBNODE _cdk_keydb_get_skblock (CDK_KEYDB_HD hd, u32 * keyid);
-CDK_KBNODE _cdk_keydb_find_bykeyid (CDK_KBNODE root, u32 * keyid);
-
 int _cdk_keydb_get_pk_byname (CDK_KEYDB_HD hd, const char * name,
                               cdkPKT_public_key ** ret_pk, int alloced);
 int _cdk_keydb_get_sk_byname (CDK_KEYDB_HD hd, const char * name,
@@ -132,14 +125,17 @@ int _cdk_stream_gets (CDK_STREAM s, char * buf, size_t count);
 int _cdk_stream_append (const char * file, CDK_STREAM * ret_s);
 int _cdk_stream_get_errno (CDK_STREAM s);
 int _cdk_stream_set_blockmode (CDK_STREAM s, size_t nbytes);
+int _cdk_stream_puts( CDK_STREAM s, const char * buf );
 
 /*-- verify.c --*/
 void _cdk_result_verify_free (Verify_Result res);
 Verify_Result _cdk_result_verify_new (void);
 
 /*-- encrypt.c --*/
-int _cdk_proc_signature_packets (CDK_HD hd, GCRY_MD_HD md,
-                                 CDK_STREAM inp, CDK_STREAM out);
+int _cdk_check_args( int overwrite, const char * in, const char * out );
+int _cdk_proc_packets( CDK_HD hd, CDK_STREAM inp, const char * output,
+                       GCRY_MD_HD md );
+
 /*-- read-packet.c --*/
 size_t _cdk_pkt_read_len (FILE * inp, int * ret_partial);
 
