@@ -1,3 +1,4 @@
+/* -*- Mode: C; c-file-style: "bsd" -*- */
 /* write-packet.c - Write OpenPGP packets
  *        Copyright (C) 2001, 2002 Timo Schulz 
  *
@@ -139,7 +140,9 @@ pkt_encode_len (CDK_STREAM out, size_t pktlen)
   if (!out)
     return CDK_Inv_Value;
 
-  if (pktlen < 192)
+  if (!pktlen)
+    return 0;
+  else if (pktlen < 192)
     rc = stream_putc (out, pktlen);
   else if (pktlen < 8384)
     {
@@ -510,7 +513,7 @@ write_public_key (CDK_STREAM out, cdkPKT_public_key * pk,
   if (!rc && pk->version < 4)
     {
       if (pk->expiredate)
-        ndays = (u16)((pk->expiredate - pk->timestamp) / 86400L);
+        ndays = (u16) ((pk->expiredate - pk->timestamp) / 86400L);
       rc = write_16 (out, ndays);
     }
   if (!rc)
@@ -535,6 +538,7 @@ calc_s2k_size (cdkPKT_secret_key * sk)
     case 3: nbytes = 11; break;
     }
   nbytes += sk->protect.ivlen;
+  nbytes++; /* single cipher byte */
   return nbytes;
 }
 
@@ -545,7 +549,7 @@ write_secret_key (CDK_STREAM out, cdkPKT_secret_key * sk,
 {
   cdkPKT_public_key * pk = NULL;
   size_t size = 6, npkey, nskey;
-  int pkttype;
+  int pkttype, s2k_mode;
   int rc = 0;
 
   if (!out || !sk || !sk->pk)
@@ -562,7 +566,7 @@ write_secret_key (CDK_STREAM out, cdkPKT_secret_key * sk,
   nskey = cdk_pk_get_nskey (pk->pubkey_algo);
   if (pk->version < 4)
     size += 2;
-  size += (1 + calc_s2k_size (sk) + 2);
+  size = !sk->is_protected? size + 3 : size + 1 + calc_s2k_size (sk) + 20;
   size += calc_mpi_size (pk->mpi, npkey);
   size += calc_mpi_size (sk->mpi, nskey);
 
@@ -589,21 +593,25 @@ write_secret_key (CDK_STREAM out, cdkPKT_secret_key * sk,
     {
       if (is_RSA (pk->pubkey_algo) && pk->version < 4)
 	stream_putc (out, sk->protect.algo);
-      else
+      else if (sk->protect.s2k)
 	{
-	  rc = stream_putc (out, 0xff);
+          s2k_mode = sk->protect.s2k->mode;
+	  rc = stream_putc (out, sk->protect.sha1chk? 0xfe : 0xff);
+          if (!rc)
+            rc = stream_putc (out, sk->protect.algo);
 	  if (!rc)
 	    rc = stream_putc (out, sk->protect.s2k->mode);
 	  if (!rc)
 	    rc = stream_putc (out, sk->protect.s2k->hash_algo);
-	  if (!rc && (sk->protect.s2k->mode == 1
-                      || sk->protect.s2k->mode == 3))
+	  if (!rc && (s2k_mode == 1 || s2k_mode == 3))
 	    {
 	      rc = stream_write (out, sk->protect.s2k->salt, 8);
-	      if (!rc && sk->protect.s2k->mode == 3)
+	      if (!rc && s2k_mode == 3)
 		rc = stream_putc (out, sk->protect.s2k->count);
 	    }
 	}
+      else
+        return CDK_Inv_Value;
       rc = stream_write (out, sk->protect.iv, sk->protect.ivlen);
     }
   if (!rc && sk->is_protected && pk->version == 4)
@@ -618,7 +626,7 @@ write_secret_key (CDK_STREAM out, cdkPKT_secret_key * sk,
       if (!rc)
         {
           if (!sk->csum)
-            sk->csum = _cdk_seckey_get_csum (sk);
+            sk->csum = _cdk_sk_get_csum (sk);
           rc = write_16 (out, sk->csum);
         }
     }
