@@ -23,13 +23,14 @@
 #include <string.h>
 #include <errno.h>
 #include <time.h>
+#include <stdarg.h>
 
 #include "opencdk.h"
 
 /* we need this for _cdk_stream_get_opaque */
+#include "../src/main.h"
 #include "../src/filters.h"
 #include "../src/stream.h"
-#include "../src/main.h"
 
 
 int
@@ -51,16 +52,17 @@ stream_test_filters (int literal, int compress, int armor, int encrypt)
     cdk_stream_set_armor_flag (s, CDK_ARMOR_MESSAGE);
   if (encrypt)
     {
-      rc = cdk_dek_new (&dek, GCRY_CIPHER_TWOFISH);
+      rc = cdk_dek_new (&dek);
       if (rc)
         {
           cdk_stream_close (s);
           return rc;
         }
+      cdk_dek_set_cipher( dek, GCRY_CIPHER_TWOFISH );
       cdk_stream_set_cipher_flag (s, dek, 0);
     }
   if (compress)
-    cdk_stream_set_compress_flag (s, CDK_COMPRESS_ZLIB);
+    cdk_stream_set_compress_flag (s, CDK_COMPRESS_ZLIB, 5);
   if (literal)
     cdk_stream_set_literal_flag (s, 0, "test.asc");
 
@@ -286,8 +288,8 @@ get_sig_time (unsigned long timestamp)
   struct tm * tmbuf;
   
   tmbuf = localtime (&timestamp);
-  snprintf (buf, sizeof buf-1, "%04d-%02d-%02d",
-            tmbuf->tm_year+1900, tmbuf->tm_mon+1, tmbuf->tm_mday);
+  sprintf (buf, "%04d-%02d-%02d",
+           tmbuf->tm_year+1900, tmbuf->tm_mon+1, tmbuf->tm_mday);
   return buf;
 }
 
@@ -444,7 +446,7 @@ stream_clearsign (char *file)
   cdk_handle_new (&hd);
   cdk_keydb_new (&db, CDK_DBTYPE_SK_KEYRING, "sec.gpg", 7);
   cdk_strlist_add (&locusr, "twoaday@daredevil");
-  rc = cdk_file_clearsign (hd, locusr, file, "out.asc");
+      //rc = cdk_file_clearsign (hd, locusr, file, "out.asc");
   if (rc)
     printf ("clearsign_file %s: `%s'\n", file, cdk_strerror (rc));
   cdk_handle_free (hd);
